DAT programName  byte "NordicDemoDual131126b", 0
CON{{

  ******* Public Notes *******

  This demo uses two Noridic nRF24L01+ modules.

  The "TRIGGER_PIN" and "TARGET_IO_PIN" do not currently work.

  You should be able to set "TARGET_IO_PIN" to -1 but I haven't tested
  this lately.

  This program can control two Nordic transceivers. Alternatively
  the one transciever could be used with conjunction with a second
  Propeller/Nordic module combo.

  If two different Propellers are used, make sure one is designated
  the master (#0) and the other the slave (#1). The Propeller will
  stop any unused cogs if only one transceiver is attached.
  
  
}}
{
  ******* Private Notes *******
  Modified from NordicPlusPlusDemo120521b

  22f Change child object.
  23b Slave received first transmission and transmitted reply
  but then neither unit received any other replies.
  23e Used "SetConfigure" method after Slave transmits data
  to get it back into receive mode again.
  23f Try the same trick for Master.
  23g Use global datapipe for now.
  24b Seems to be working pretty well. Both master
  and slave are transmitting and receiving.
  24c For some reason this version doesn't work.
  I don't understand why. I just removed a few
  debugging lines.
  24d The debug lines didn't matter. It was the pause
  that was required. Why?
  24e It can appear that the master receives a message
  twice in a row if the rx buffer isn't checked soon enough
  after the first transmission.
  24f Fix math exchange part.
  24f Works on two separate Propeller chips.
  24g Clean up a bit.
  26b More cleaning up and delete many of these notes.
  
}
 
CON
  _clkmode = xtal1 + pll16x                           
  _xinfreq = 5_000_000
    
CON


  'Pins to Nordic0 nRF24L01+
 { Nordic0Base = 0
  
  Nordic0_CE = Nordic0Base + 5
  Nordic0_CSN = Nordic0Base + 4
  Nordic0_SCK = Nordic0Base + 3
  Nordic0_MOSI = Nordic0Base + 2
  Nordic0_MISO = Nordic0Base + 1
  Nordic0_IRQ = Nordic0Base + 0

  'Pins to Nordic1 nRF24L01+
  Nordic1Base = 6
  
  Nordic1_CE = Nordic1Base + 5
  Nordic1_CSN = Nordic1Base + 4
  Nordic1_SCK = Nordic1Base + 3
  Nordic1_MOSI = Nordic1Base + 2
  Nordic1_MISO = Nordic1Base + 1
  Nordic1_IRQ = Nordic1Base + 0    }

    'Pins to Nordic0 nRF24L01+
  Nordic0Base = 0
  
  Nordic0_CE = Nordic0Base + 0
  Nordic0_CSN = Nordic0Base + 2
  Nordic0_SCK = Nordic0Base + 4
  Nordic0_MOSI = Nordic0Base + 6
  Nordic0_MISO = Nordic0Base + 8
  Nordic0_IRQ = Nordic0Base + 10

  'Pins to Nordic1 nRF24L01+
  Nordic1Base = 1
  
  Nordic1_CE = Nordic1Base + 0
  Nordic1_CSN = Nordic1Base + 2
  Nordic1_SCK = Nordic1Base + 4
  Nordic1_MOSI = Nordic1Base + 6
  Nordic1_MISO = Nordic1Base + 8
  Nordic1_IRQ = Nordic1Base + 10

CON

  _TimerLed = 13

  TRIGGER_PIN = 14
  TARGET_IO_PIN = 12

  _TriggerByte =$FF

  DEGUB_BAUD = 115200

  _MaxDisplayInterval = 25
  _InitialDisplayInterval = 3
  _IncrementDisplayInterval = 6
  _TimesToDisplayMenu = 4

  DEFAULT_DATAPIPE = 1
  
VAR

  long nordicStack[100], debugLock, txTime, rxTime, intervalTime
  long triggerCount, previousTriggerCount, targetActivityCount, previousTargetActivityCount
  long rxCount[2], previousRxCount[2]
  long timeoutInterval, bigBufferPtr, rxTimer
  long oneSecond', displayInterval, displayTimer
  long masterTxInterval, txTimer
  long activeMessagePtrM, activeMessagePtrS
  'long txTimerS, slaveTxInterval
  long globalDatapipe, previousGlobalDatapipe[2]
  
  word sumOfBytes

  byte txPayload0[Nordic#LARGEST_PAYLOAD_SIZE + 1] ' Add 1 so a terminating zero may be added if desired.
  byte txPayload1[Nordic#LARGEST_PAYLOAD_SIZE + 1]
  byte rxPayload0[Nordic#LARGEST_PAYLOAD_SIZE + 1] 
  byte rxPayload1[Nordic#LARGEST_PAYLOAD_SIZE + 1]
  byte addressSize
  byte nordicStartingFlag
  byte menuCount
  byte bytesToAdd[2]
  byte bytesToAddS[2], rxDatapipe[2]
  
OBJ   

  Debug : "FullDuplexSerialPlusMultiObj"   
  Nordic[2] : "Nordic131126a"
  'Timer: "FlashTimer120603a"
  
PUB Demo | localIndex



  txPayload0[0] := _TriggerByte
  txPayload1[0] := _TriggerByte
  globalDatapipe := DEFAULT_DATAPIPE
  previousGlobalDatapipe[0] := -1
  previousGlobalDatapipe[1] := -1  ' datapipes are used yet
  
  oneSecond := clkfreq
  'timeoutInterval := oneSecond * TIMEOUT_INTERVAL
  
  addressSize := byte[@configNordic + Nordic#SETUP_AW_OFFSET] 
  addressSize += 2
  addressSize <#= 5
  addressSize #>= 3            

  Debug.start(31, 30, 0, DEGUB_BAUD)
  debugLock := locknew
  waitcnt(clkfreq * 2 + cnt)   ' time to open terminal window
  repeat until not lockset(debugLock)
  DebugStr(String(13, "Start of Nordic Demo"))
  DebugStr(String(13, "programName = "))
  Debug.str(@programName)
  DebugStr(String(13, "name of Nordic SPI object = "))
  Debug.str(Nordic.GetObjectName)

  Debugstr(String(13, "About to launch SlaveNordic."))
  lockclr(debugLock)
  nordicStartingFlag := 1
  result := cognew(SlaveNordic, @nordicStack)
  repeat while nordicStartingFlag == 1 'Don't let both Nordic drives start at once
  repeat until not lockset(debugLock)
  Debugstr(String(13, "After launch of SlaveNordic."))
  lockclr(debugLock)

  MasterNordic

PUB MasterNordic | localIndex', dataPipe, previousPipe

  'Initialize Nordic nRF24L01

  repeat until not lockset(debugLock)
  Debugstr(String(13, "Beginning of MasterNordic Method"))
  lockclr(debugLock) 
  Nordic[0].Init(@rxPayload0, @txPayload0)      

  result := Nordic[0].Start(Nordic0_IRQ, Nordic0_MISO, Nordic0_MOSI, Nordic0_SCK, {
  } Nordic0_CSN, Nordic0_CE, -1, TARGET_IO_PIN, @configNordic)
  
  repeat until not lockset(debugLock) 
  Debugstr(String(13, "Master SPI started on cog # "))
  Debug.dec(result)
  Debugstr(String(13, "Master Nordic Started", 13))
  Debugstr(String("rx address ="))
  repeat localIndex from 0 to addressSize - 1 
    Debug.str(String(" $"))
    Debug.hex(rxAddrP0[localIndex], 2)
  Debugstr(String(13, "tx address ="))
  repeat localIndex from 0 to addressSize - 1 
    Debug.str(String(" $"))
    Debug.hex(txAddr[localIndex], 2)
  nordicStartingFlag := 0
   
  lockclr(debugLock) 

  activeMessagePtrM := @packetTemplateM

  localIndex := 0

  masterTxInterval := clkfreq'* 3 '2

  repeat until not lockset(debugLock)
  Debugstr(String(13, "intial"))
  result := DisplayConfig(0) 
  
  lockclr(debugLock)

  if result == 0
    repeat until not lockset(debugLock)
    Debugstr(String(13, "A master Nordic module was not found."))
    Debugstr(String(13, "Stopping SPI driver cog and this cog #"))
    Debug.Dec(cogid)
    lockclr(debugLock)
    'waitcnt(clkfreq + cnt)
    Nordic[0].Stop
    cogstop(cogid)
      
  txTimer := cnt
  repeat
    rxCount[0] := Nordic[0].ReadRxCounts
    if rxCount[0] <> previousRxCount[0]
      {repeat until not lockset(debugLock)
      Debugstr(String(13, "Master has received a new transmission."))
      Debugstr(String(13, "rxCount[0] = "))
      Debug.dec(rxCount[0])
      Debugstr(String(13, "previousRxCount[0] = "))
      Debug.dec(previousRxCount[0])   }
      result := Nordic[0].ReadStatus
      'Debugstr(String(13, "status = %"))
      'Debug.bin(result, 8)
      'lockclr(debugLock)
      
      result := DisplayPayload(0, result)
   
      previousRxCount[0]++
      PreparePacketM
      Nordic[0].TransmitStringOnDataPipe(globalDatapipe, activeMessagePtrM)
      Nordic[0].SetConfigure 
      txTimer := cnt
     
    elseif cnt - txTimer > masterTxInterval
      'txTimer += masterTxInterval
      txTimer := cnt

      Nordic[0].TransmitStringOnDataPipe(globalDatapipe, activeMessagePtrM)
      previousGlobalDatapipe[0] := globalDatapipe
     
      Nordic[0].SetConfigure
     

PUB PreparePacketM | localIndex

  Hex(bytesToAdd[0]++, 2, @packetTemplateM + 12)
  bytesToAdd[1]++ ' The second byte will increament by 2
  Hex(bytesToAdd[1]++, 2, @packetTemplateM + 18)

PUB Hex(value, digits, localPtr)
'' ripped from FullDuplexSerial
  value <<= (8 - digits) << 2
  repeat digits
    byte[localPtr++] := lookupz((value <-= 4) & $F : "0".."9", "A".."F")
              
PUB SlaveNordic | localIndex, txFlag', dataPipe, previousPipe

  repeat until not lockset(debugLock)
  Debugstr(String(13, "SlaveNordic cog started."))
  lockclr(debugLock)
  'Initialize Nordic nRF24L01
  Nordic[1].Init(@rxPayload1, @txPayload1)      
  result := Nordic[1].Start(Nordic1_IRQ, Nordic1_MISO, Nordic1_MOSI, Nordic1_SCK, {
  } Nordic1_CSN, Nordic1_CE, -1, TARGET_IO_PIN, @configNordic)   
  
 
  repeat until not lockset(debugLock)
  Debugstr(String(13, "Slave SPI started on cog # "))
  Debug.dec(result)
  
  Debugstr(String(13, "Slave Nordic Started"))
  Debugstr(String(13, "rx address ="))
  repeat localIndex from 0 to addressSize - 1
    Debug.str(String(" $"))
    Debug.hex(rxAddrP0[localIndex], 2)
  Debugstr(String(13, "tx address ="))
  repeat localIndex from 0 to addressSize - 1    
    Debug.str(String(" $"))
    Debug.hex(txAddr[localIndex], 2)
  lockclr(debugLock)
  nordicStartingFlag := 0

  'dataPipe := DEFAULT_DATAPIPE
  'previousPipe := -1 ' some impossible number
  
  activeMessagePtrS := @packetTemplateS
  'Nordic[1].SetDataPipe(dataPipe, activeMessagePtrS) ' This will block this cog from receiving.
  
  txFlag := 0

  repeat until not lockset(debugLock)
  result := DisplayConfig(1)
  
  lockclr(debugLock)

  if result == 0
    repeat until not lockset(debugLock)
    Debugstr(String(13, "A slave Nordic module was not found."))
    Debugstr(String(13, "Stopping SPI driver cog and this cog #"))
    Debug.Dec(cogid)
    lockclr(debugLock)
    Nordic[1].Stop
    cogstop(cogid)
  repeat

    rxCount[1] := Nordic[1].ReadRxCounts
 
    if rxCount[1] <> previousRxCount[1]
      
      {repeat until not lockset(debugLock)
      Debugstr(String(13, "Slave has received a new transmission."))
      Debugstr(String(13, "rxCount[1] = "))
      Debug.dec(rxCount[1])
      Debugstr(String(13, "previousRxCount[1] = "))
      Debug.dec(previousRxCount[1])   }
      result := Nordic[1].ReadStatus
      'Debugstr(String(13, "status = %"))
      'Debug.bin(result, 8)
      'lockclr(debugLock)
      result := DisplayPayload(1, result)
      Nordic[1].SetConfigure
      if rxPayload1[11] == "$" and rxPayload1[15] == "+" and rxPayload1[17] == "$"
        txFlag := 1
        {repeat until not lockset(debugLock)
        Debugstr(String(13, "New transmission is a math problem. **********************"))  
        Debugstr(String(13, "Sent to master: "))
        Debugstr(@packetTemplateS)
        lockclr(debugLock) }
        PreparePacketS
        'activeMessagePtrS := @packetTemplateS
        Nordic[1].TransmitStringOnDataPipe(globalDatapipe, @packetTemplateS)
        Nordic[1].SetConfigure ' This is needed to receive again (it shouldn't be needed)
      else
   
        repeat until not lockset(debugLock)   
        Debugstr(String(13, "Not math."))
        lockclr(debugLock)         
      previousRxCount[1]++
  
PUB DisplayConfig(localNordic)

  if localNordic
    Nordic[1].CmdRead(Nordic#CONFIG, @rxPayload1, 1)
    Debugstr(String(13, "Slave config register = $"))
    Debug.Hex(rxPayload1[0], 2)
    Debugstr(String(13, "Slave config register = %"))
    Debug.bin(rxPayload1[0], 8)
    result := rxPayload1[0]
  else
    Nordic[0].CmdRead(Nordic#CONFIG, @rxPayload0, 1)
    Debugstr(String(13, "Master config register = $"))
    Debug.Hex(rxPayload0[0], 2)
    Debugstr(String(13, "Master config register = %"))
    Debug.bin(rxPayload0[0], 8)
    result := rxPayload0[0]

PUB PreparePacketS | localIndex

  bytesToAddS[0] := FromAsciiHex(2, @rxPayload1 + 12)
  bytesToAddS[1] := FromAsciiHex(2, @rxPayload1 + 18)
  sumOfBytes := bytesToAdd[0] + bytesToAddS[1]
  Hex(bytesToAddS[0], 2, @packetTemplateS + 1)
  Hex(bytesToAddS[1], 2, @packetTemplateS + 7)
  Hex(sumOfBytes, 4, @packetTemplateS + 13)
    
PUB FromAsciiHex(digits, localPtr) | localCharacter  
'' not ripped hence suboptimal
  repeat digits
    localCharacter := byte[localPtr++]
    if localCharacter => "0" and localCharacter =< "9"
      localCharacter -= "0"
    elseif localCharacter => "A" and localCharacter =< "F"
      localCharacter -= "A"
      localCharacter += $0A
    else
      localCharacter := 0
    result <<= 4
    result += localCharacter
     
{PUB DisplayMenu

  repeat until not lockset(debugLock)
  Debugstr(@menu0)
  Debug.Dec(Nordic#TRIGGER_DATAPIPE)
  Debugstr(@menu1)
  Debug.Dec(Nordic#TRIGGER_DATAPIPE)
  Debugstr(@menu2)
  Debug.Dec(Nordic#TRIGGER_DATAPIPE)
  Debugstr(@menu3)
  Debug.Dec(TRIGGER_PIN)
  Debugstr(@menu4)
  lockclr(debugLock)
  'displayTimer := cnt
}
PUB DisplayPayload(nordicId, localStatus) | activeBuffer

  localStatus &= %00001110 ' leave only data line info
  localStatus >>= 1             ' shift data line down one

  if nordicId
    activeBuffer := @rxPayload1
  else
    activeBuffer := @rxPayload0
  repeat until not lockset(debugLock)
  Debugstr(String(13, "Nordic #"))
  Debug.dec(nordicId)
  if localStatus == Nordic#RX_FIFO_EMPTY
    Debugstr(String(": RX FIFO Empty"))
  else
    rxDatapipe[nordicId] := localStatus
    'Debugstr(String(" has received a packet on data pipe #"))
    'Debug.dec(localStatus) 
    Debugstr(String(" payload = ", 34))
    Debugstr(activeBuffer)
    Debug.tx(34)
  lockclr(debugLock)
  'Nordic[nordicId].SetConfigure
  
  result := localStatus        

PUB DebugSafe(localPtr, localSize)
'' Use this method if there may be non-printable
'' characters in message.

  if debugLock == -1 ' allows debug feature to be turned off
    return

  if byte[localPtr] == 13
    Debug.tx(13)
    Debug.dec(cogid)  ' Identifies which cog is sending message
    Debug.tx(":")
    Debug.tx(32)
    localPtr++
    localSize--

  repeat localSize
    DebugTx(byte[localPtr++])
      
PUB DebugStr(localPtr)
'' display cog ID at the beginning of debug statements

  if byte[localPtr] == 13
    Debug.tx(13)
    Debug.dec(cogid)
    Debug.tx(":")
    Debug.tx(32)
    localPtr++
  Debug.Str(localPtr)  

PUB DebugTx(localCharacter)
'' Non-printable ASCII characters will be displayed as
'' hex.

  'if localCharacter => " " and localCharacter =< "~"
  if localCharacter => 32 and localCharacter =< 126
    Debug.tx(localCharacter)
  else
    Debug.tx("<")
    Debug.tx("$")
    Debug.hex(localCharacter, 2)
    Debug.tx(">")
                     
DAT
                                       '1         2         3
                             '01234567890123456789012345678901
packetTemplateM         byte "Please add $00 + $00", 0[Nordic#LARGEST_PAYLOAD_SIZE - 19]
packetTemplateS         byte "$00 + $00 = $0000   ", 0[Nordic#LARGEST_PAYLOAD_SIZE - 19]
                             ' zeros will be replaced my program

DAT

txAddress     byte $E7[5]
txAddress1    byte $C2[5]
txAddress2    byte $C3, $C2[4] '$C2[4], $C3  ' maybe this should be byte  
txAddress3    byte $C2[4], $C4
txAddress4    byte $C2[4], $C5
txAddress5    byte $C2[4], $C6

testAddress   byte $AA, $BB, $CC, $DD, $EE
  
''------------------ Cofig settings for nRF24L01+ modules ----------------------------------------
'' ----------------- Keep below together and in order ------------------------------------------------
configNordic  byte %00111111 '%00111011  Register $00
              ' use %00111111($3F)(2 bytes CRC) or %00111011($3B)(1 byte CRC)
              ' when communicating with older nRF2401A modules
              ' make sure this matches nRF2401A setting
              ' %00001111($0F) future with more irq options
              ' 7 only zero allowed, 6 Mask irq by RX_DR (1 = no irq), 5 Mask TX_DS, 4 Mask MAX_RT,
              ' 3 Enable CRC, 2 CRC size (1 = 2 bytes), 1 1 = power up, 0 1 = PRX 0 = PTX
              ' RX_DR Receive Data Ready
              ' TX_DS Transmit Data Sent
              ' MAX_RT Maximum Retries
              byte $00 '$03 ' Register $01 Auto Acknowledge
              ' use $00 when communicating with older nRF2401A modules
              ' %00111111 future
              ' 7:6 only %00, enable pipe 5 through 0
              byte %00111111 ' Register $02
              ' 7:6 only zero allowed, 5:0 enable rx addresses pipe 5:0
              byte $03 ' Register $03 SETUP_AW setup Address Widths 
              ' 7:2 only %000000, 1:0 %01 = 3 %11 = 5
              byte $00 ' Register $04 SETUP_RETR Setup Auto Retransmission
              ' use $00 when communicating with older nRF2401A modules     
              ' %00101111 = 750us delay to retx, tx 15 tries
              ' %01001111 = 1250us delay to retx, tx 15 tries
              ' 7:4 ARD delay 1111 = 4000us, 3:0 retry count 0 - 15 (1111)
              byte $02 ' Register $05 RF_CH RF Channel
              ' 7 leave 0, 6:0 frequency channel
              ' F = (2400 + RF_CH)MHz (both nRF24L01+ and nRF2401A modules)
              byte $0E ' $06 ' Register $06 RF_SETUP RF Setup        ' use $0E for 2Mbps
              ' don't use 2Mbps when communicating with older nRF2401A modules
              ' $06 = 1Mbps and full power
              ' 7:6 %00 only, 5 RF_DR_LOW 1 = 250kbps use 0 for higher speeds, 4 leave %0
              ' 3 1 = 2Mbps 0 = 1Mbps, 2:1 RF_POWER %11 = 0dBm (highest power), 0 don't care
              ' Register $07 STATUS Status Register used below.
              ' Register $08 OBSERVE_TX 7:4 lost packet count (read only),
              ' 3:0 retransmitted packets count (read only).  I plan to use this
              ' register's infomation in the future.
              ' Register $09 RPD 7:1 reserved (read only), 0 Received Power Detector (read only)
              ' Register $0A RX_ADDR_P0 39:0 Receive address pipe 0, 5 bytes maximum LSByte written first
              ' used below rxAddress0 and rxAddress1
              ' Register $0B RX_ADDR_P1 39:0 Receive address pipe 1
              ' Register $0C RX_ADDR_P2 7:0 Receive address pipe 2
              ' Register $0D RX_ADDR_P3 7:0 Receive address pipe 3
              ' Register $0E RX_ADDR_P4 7:0 Receive address pipe 4
              ' Register $0F RX_ADDR_P5 7:0 Receive address pipe 5
              ' Register $10 TX_ADDR 39:0 Transmit address
'configNordic[7]
nordicStatus  byte %01110000 ' $40   %01110000   '$70   'Register $07 STATUS
              ' 7 only zero allowed, 6 Data Ready RX, 5 TX ACK received, MAX_RT failed,
              ' 3:1 Data pipe with payload %111 if empty (read only), 0 1= TX FIFO full (read only)

              byte 0 'Register $08 OBSERV_TX Transmit observe
              byte 0 'Register $09 RPD Received Power Detector / Carrier Detect in nFR24L01
rxAddrP0      byte $E7[5] 'Register $0A RX_ADDR_P0
rxAddrP1      byte $C2[5] 'Register $0B RX_ADDR_P1
              '' Always use five bytes in these two memory locations so register day will be
              '' in the correct location of hub RAM.
              
rxAddrP2      byte $C3 'Register $0C RX_ADDR_P2
rxAddrP3      byte $C4 'Register $0D RX_ADDR_P3
rxAddrP4      byte $C5 'Register $0E RX_ADDR_P4
rxAddrP5      byte $C6 'Register $0F RX_ADDR_P5
txAddr        byte $E7[5] 'Register $10 TX_ADDR $C2[5] '
              '' Always use five bytes in in memory location so register day will be
              '' in the correct location of hub RAM.
              
payloadSizeP0 byte 32 'Register $11 RX_PW_P0
              ' This will need to be reduced to 25 (assuming 2 bytes CRC and 5 bytes address)
              ' to work with older nRF2401 modules.
              ' Newer nRF24L01+ modules can have payloads as large as 32 bytes.
              ' 7:6 only zero allowed, 5:0 Number of bytes in RX payload in data pipe 0
              ' (1 to 32)(0 = pipe not used)
payloadSizeP1 byte 32 'Register $12 RX_PW_P1 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 1
payloadSizeP2 byte 32 'Register $13 RX_PW_P2 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 2
payloadSizeP3 byte 32 'Register $14 RX_PW_P3 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 3
payloadSizeP4 byte 32 'Register $15 RX_PW_P4 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 4
payloadSizeP5 byte 1 'Register $16 RX_PW_P5 7:6 only zero allowed,
              ' 5:0 Number of bytes in RX payload in data pipe 5
              byte 0 'Register $17 FIFO_STATUS 7 only zero allowed, 6 TX_REUSE (read only), 
              ' 5 TX_FULL %1 = full (read only), 4 TX_EMPTY %1 = empty (read only),
              ' 3:2 only zero allowed, 1 RX_FULL %1 = full (read only),
              ' 0 RX_EMPTY %1 = empty (read only)
              byte 0 'Register $1C DYNPD Enable dynamic payload length, 7:6 only zero allowed,
              ' 5 DPL_P5 Enable dynamic payload length data pipe 5 (Requires EN_DPL and ENAA_5)
              ' 4 DPL_P4 Enable dynamic payload length data pipe 4 (Requires EN_DPL and ENAA_4)
              ' 3 DPL_P3 Enable dynamic payload length data pipe 3 (Requires EN_DPL and ENAA_3)
              ' 2 DPL_P2 Enable dynamic payload length data pipe 2 (Requires EN_DPL and ENAA_2)
              ' 1 DPL_P1 Enable dynamic payload length data pipe 1 (Requires EN_DPL and ENAA_1)
              ' 0 DPL_P0 Enable dynamic payload length data pipe 0 (Requires EN_DPL and ENAA_0)
              byte 0 'Register $1D FEATURE Feature Register, 1 EN_ACK_PAY Enables Payload with ACK,
              ' 0 EN_DYN_ACK Enables the W_TX_PAYLOAD_NOACK command

' End of registers
              
'' ----------------- Keep above together and in order ------------------------------------------------             